
function getListTitle(id) {
  for (let i = 0; i < listDescriptors.length; ++i) {
    if (listDescriptors[i].settings == id) {
      return listDescriptors[i].title;
    }
  }

  return id;
}

function displayPreferences(evt) {
  let loadObj = { [PREF_BLOCK_IMAGES]: DEFAULT_BLOCK_IMAGES,
                  [PREF_LOG_INTERVENTIONS]: DEFAULT_LOG_INTERVENTIONS };

  let promise = browser.storage.local.get(loadObj);
  promise.then(function (obj) {
      let ctl = document.getElementById('blockImages');
      if (ctl) {
        ctl.checked = obj[PREF_BLOCK_IMAGES];
      }

      ctl = document.getElementById('logInterventions');
      if (ctl) {
        ctl.checked = obj[PREF_LOG_INTERVENTIONS];
      }
    });
  promise.catch(function(error) {
      console.error(DBTAG + 'Failed to load preferences due to: ' + error);
    });
}

function populateListSelect(evt) {
  let sel = document.getElementById('lists');
  if (sel) {
    for (let i = 0; i < listDescriptors.length; ++i) {
      let opt = document.createElement('option');
      opt.value = listDescriptors[i].settings;
      opt.textContent = listDescriptors[i].title;
      sel.appendChild(opt);
    }
  }
}

function dumpList(evt) {
  let list = evt.target.value;

  let ctl = document.getElementById('update');
  if (ctl) {
      ctl.disabled = list == '';
  }

  ctl = document.getElementById('listContent');
  if (ctl) {
    ctl.value = '';

    if (list) {
      let promise = browser.storage.local.get({ [list] : '' });
      promise.then(function (obj) {
          if (Array.isArray(obj[list])) {
            ctl.value = obj[list].join('\n');
          } 
        });
      promise.catch(function(error) {
          console.error(DBTAG + 'Failed to load list "' + getListTitle(list) + '" from preferences due to: ' + error);
        });
    }
  }
}

function takeoverPreferences(evt) {
  let saveObj = { [PREF_BLOCK_IMAGES]: DEFAULT_BLOCK_IMAGES,
                  [PREF_LOG_INTERVENTIONS]: DEFAULT_LOG_INTERVENTIONS };

  let ctl = document.getElementById('blockImages');
  if (ctl) {
    saveObj[PREF_BLOCK_IMAGES] = ctl.checked;
  }

  ctl = document.getElementById('logInterventions');
  if (ctl) {
    saveObj[PREF_LOG_INTERVENTIONS] = ctl.checked;
  }

  let promise = browser.storage.local.set(saveObj);
  promise.then(function () {
      console.info(DBTAG + 'Preferences saved');
    });
  promise.catch(function(error) {
      console.error(DBTAG + 'Failed to store preferences due to: ' + error);
    });
}

function insertTab(evt) {
  let ctl = document.getElementById('listContent');
  if (ctl) {
    let start = ctl.selectionStart;
    let end = ctl.selectionEnd;

    ctl.setRangeText('\t', start, end, 'end');
    ctl.focus();
  }
}

function takeoverList(evt) {
  let list = '';

  let ctl = document.getElementById('lists');
  if (ctl) {
    list = ctl.value;
  }

  if (list) {
    ctl = document.getElementById('listContent');
    if (ctl) {
      let lines = ctl.value.split('\n');
      let strings = [];

      for (let i = 0; i < lines.length; ++i) {
        let line = lines[i].trim();
        if (line.length > 0 && line.charAt(0) !== '#') {
          strings.push(line);
        }
      }

      let saveObj = { [list]: strings };
      let promise = browser.storage.local.set(saveObj);
      promise.then(function () {
          console.info(DBTAG + 'DualBlock "' + getListTitle(list) + '" list was updated');
        });
      promise.catch(function(error) {
          console.error(DBTAG + 'Failed to update list "' + getListTitle(list) + '" due to: ' + error);
        });
    }
  }
}

function handleOnLoaded(evt) {
  displayPreferences(evt);
  populateListSelect(evt);

  let ctl = document.getElementById('save');
  if (ctl) {
    ctl.addEventListener('click', takeoverPreferences);
  }

  ctl = document.getElementById('lists');
  if (ctl) {
    ctl.addEventListener('change', dumpList);
  }

  ctl = document.getElementById('tab');
  if (ctl) {
    ctl.addEventListener('click', insertTab);
  }

  ctl = document.getElementById('update');
  if (ctl) {
    ctl.addEventListener('click', takeoverList);
  }
}

document.addEventListener('DOMContentLoaded', handleOnLoaded);
